unit HistMenu;
{
  Histmenu component for Delphi 3.0.
  Copyright  1997 Wim Vissers, The Netherlands.

  Written by: Wim Vissers
  email: wvissers@dds.nl

  You are free to use, modify, and distribute this code subject to following
  restrictions:
  - If you copy portions of my code or get ideas from my code, please give
    me credit.
  - Do not release modified versions of this source code unless it has
    significant differences and is also freeware.

  Install this component by copying histmenu.pas and histmenu.dcr to a
  convenient directory, and choosing Component, Install Component from the
  Delphi menu. The THistmenu component will be added to the 'Samples' section.

  The component assists in adding 'last openened files'-like menu items to any
  mainmenu you want.
  To use this component, just add it to a form containing a MainMenu component,
  and set the MenuItem property to an existing menu-item. The history-items will
  be added using the 'Add' method, taking a caption string and an integer Tag
  as its arguments.
  To save it to the registry, make sure you have the BaseKey property filled out
  right, and call the 'RegSave' method. To restore it, call the 'RegRestore'
  method. The registry key used will be
  'HKEY_CURRENT_USER\<your base key>\<histmenu name>'.

  Misc. notes:
  - I released this component as freeware because I needed a component like
    this and it may be of interest for others, too.
    No support of any kind is given for this component, however. It is provided
    'as is', and no liability of any kind is accepted.

}


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics,
  Controls, Forms, Dialogs, Menus, Registry;

type
  THistMenuEvent = procedure(Sender:TObject; Caption:String; Tag:LongInt) of object;
  THistMenu = class(TComponent)
  private
    { Private declarations }
    FMenuItem: TMenuItem;
    FMaxHistory: integer;
    FAddSeparator: boolean;
    FItemCount: integer;
    FBaseKey: String;
    FFirstIndex: integer;
    FOnClick: THistMenuEvent;
    procedure SetMenuItem(Value: TMenuItem);
    procedure SetMaxHistory(Value: integer);
    procedure SetAddSeparator(Value: boolean);
    procedure InsertSeparator;
    procedure OnItemClick(Sender: TObject);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure Add(Caption:String; Tag: LongInt);
    procedure RegSave;
    procedure RegRestore;
  published
    { Published declarations }
    property MenuItem:TMenuItem read FMenuItem write SetMenuItem;
    property MaxHistory:integer read FMaxHistory write SetMaxHistory default 5;
    property AddSeparator:boolean read FAddSeparator write SetAddSeparator default False;
    property ItemCount:integer read FItemCount;
    property BaseKey:String read FBaseKey write FBaseKey;
    property OnClick:THistMenuEvent read FOnClick write FOnClick;
  end;

procedure Register;

implementation

procedure THistMenu.SetMenuItem(Value: TMenuItem);
begin
  if Assigned(FMenuItem) then
    while FMenuItem.Count > FFirstIndex do
      FMenuItem.Remove(FMenuItem.Items[FMenuItem.Count-1]);
  FMenuItem := Value;
  FFirstIndex := FMenuItem.Count;
end;

procedure THistMenu.SetMaxHistory(Value: integer);
begin
  if Value > 1 then FMaxHistory := Value;
end;

procedure THistMenu.SetAddSeparator(Value: boolean);
begin
  if Value <> FAddSeparator then
  begin
    FAddSeparator := Value;
    if Assigned(FMenuItem) and (FItemCount > 0) then
    begin
      if FAddSeparator then
      begin
        InsertSeparator;
      end else begin
        Dec(FFirstIndex);
        FMenuItem.Delete(FFirstIndex);
      end;
    end;
  end;
end;

procedure THistMenu.InsertSeparator;
var
  Sep: TMenuItem;
begin
  Sep := TMenuItem.Create(FMenuItem);
  Sep.Caption := '-';
  FMenuItem.Insert(FFirstIndex, Sep);
  Inc(FFirstIndex);
end;

procedure THistMenu.OnItemClick(Sender: TObject);
begin
  if Assigned(FOnClick) then
   FOnClick(Sender, (Sender as TMenuItem).Caption,
     (Sender as TMenuItem).Tag);
end;

constructor THistMenu.Create(AOwner: TComponent);
begin
  inherited;
  FItemCount := 0;
  FMaxHistory := 5;
end;

procedure THistMenu.Add(Caption:String; Tag: LongInt);
var
  Itm: TMenuItem;
begin
  if not Assigned(FMenuItem) then Exit;
  if (FItemCount = 0) and (FAddSeparator) then
    InsertSeparator;
  if FItemCount >= FMaxHistory then
    FMenuItem.Delete(FFirstIndex)
  else
    Inc(FItemCount);
  Itm := TMenuItem.Create(FMenuItem);
  Itm.OnClick := OnItemClick;
  Itm.Caption := Caption;
  Itm.Tag := Tag;
  FMenuItem.Add(Itm);
end;

procedure THistMenu.RegSave;
var
  Reg:TRegistry;
  i: integer;
begin
  if BaseKey <> '' then
  begin
    Reg := TRegistry.Create;
    try
      if Reg.OpenKey(BaseKey + '\' + Name, True) then
      begin
        for i := 0 to FItemCount - 1 do
        begin
          Reg.WriteString('C'+IntToStr(i),
            FMenuItem.Items[i + FFirstIndex].Caption);
          Reg.WriteInteger('T'+IntToStr(i),
            FMenuItem.Items[i + FFirstIndex].Tag);
        end;
      end;
    finally
      Reg.Free;
    end;
  end;
end;

procedure THistMenu.RegRestore;
var
  Reg:TRegistry;
  i:integer;
begin
  if BaseKey <> '' then
  begin
    Reg := TRegistry.Create;
    try
      if Reg.OpenKey(BaseKey + '\' + Name, False) then
      begin
        i := 0;
        while Reg.ValueExists('C'+IntToStr(i)) do
        begin
          Add(Reg.ReadString('C'+IntToStr(i)),
            Reg.ReadInteger('T'+IntToStr(i)));
          Inc(i);
        end;
      end;
    finally
      Reg.Free;
    end;
  end;
end;

procedure Register;
begin
  RegisterComponents('Samples', [THistMenu]);
end;

end.
